@@ -142,15 +142,15 @@ def resize_video_to(
     mode = 'nearest'
 ):
     orig_video_size = video.shape[-1]
+
     frames = video.shape[2]
-
-    if orig_video_size == target_image_size and default(target_frames, frames) == frames:
-        return video
-
     target_frames = default(target_frames, frames)
 
     target_shape = (target_frames, target_image_size, target_image_size)
 
+    if tuple(video.shape[-3:]) == target_shape:
+        return video
+
     out = F.interpolate(video, target_shape, mode = mode)
 
     if exists(clamp_range):
@@ -158,6 +158,28 @@ def resize_video_to(
         
     return out
 
+def scale_video_time(
+    video,
+    downsample_scale = 1,
+    mode = 'nearest'
+):
+    if downsample_scale == 1:
+        return video
+
+    image_size, frames = video.shape[-1], video.shape[-3]
+    assert divisible_by(frames, downsample_scale), f'trying to temporally downsample a conditioning video frames of length {frames} by {downsample_scale}, however it is not neatly divisible'
+
+    target_frames = frames // downsample_scale
+
+    resized_video = resize_video_to(
+        video,
+        image_size,
+        target_frames = target_frames,
+        mode = mode
+    )
+
+    return resized_video
+
 # classifier free guidance functions
 
 def prob_mask_like(shape, prob, device):
@@ -1599,6 +1621,7 @@ class Unet3D(nn.Module):
         text_mask = None,
         cond_images = None,
         cond_video_frames = None,
+        post_cond_video_frames = None,
         self_cond = None,
         cond_drop_prob = 0.,
         ignore_time = False
@@ -1627,8 +1650,11 @@ class Unet3D(nn.Module):
                 lowres_cond_img = torch.cat((cond_video_frames, lowres_cond_img), dim = 2)
                 cond_video_frames = torch.cat((cond_video_frames, cond_video_frames), dim = 1)
 
+            if exists(post_cond_video_frames):
+                lowres_cond_img = torch.cat((lowres_cond_img, post_cond_video_frames), dim = 2)
+                post_cond_video_frames = torch.cat((post_cond_video_frames, post_cond_video_frames), dim = 1)
+
         # conditioning on video frames as a prompt
-        # todo - add post_cond_video_frames as well
 
         num_preceding_frames = 0
         if exists(cond_video_frames):
@@ -1641,6 +1667,19 @@ class Unet3D(nn.Module):
 
             num_preceding_frames = cond_video_frames_len
 
+        # conditioning on video frames as a prompt
+
+        num_succeeding_frames = 0
+        if exists(post_cond_video_frames):
+            cond_video_frames_len = post_cond_video_frames.shape[2]
+
+            assert divisible_by(cond_video_frames_len, self.total_temporal_divisor)
+
+            post_cond_video_frames = resize_video_to(post_cond_video_frames, x.shape[-1])
+            x = torch.cat((post_cond_video_frames, x), dim = 2)
+
+            num_succeeding_frames = cond_video_frames_len
+
         # condition on input image
 
         assert not (self.has_cond_image ^ exists(cond_images)), 'you either requested to condition on an image on the unet, but the conditioning image is not supplied, or vice versa'
@@ -1853,4 +1892,7 @@ class Unet3D(nn.Module):
         if num_preceding_frames > 0:
             out = out[:, :, num_preceding_frames:]
 
+        if num_succeeding_frames > 0:
+            out = out[:, :, :-num_succeeding_frames]
+
         return out
